@@ -1,5 +1,5 @@
-# coding: utf8
-# $Id: __init__.py 7538 2012-11-23 01:18:49Z milde $
+# coding: utf-8
+# $Id: __init__.py 7668 2013-06-04 12:46:30Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -17,7 +17,7 @@ import warnings
 import unicodedata
 from docutils import ApplicationError, DataError
 from docutils import nodes
-from docutils.io import FileOutput
+import docutils.io
 from docutils.utils.error_reporting import ErrorOutput, SafeString
 
 
@@ -513,8 +513,28 @@ def get_stylesheet_list(settings):
     stylesheets = settings.stylesheet_path or settings.stylesheet or []
     # programmatically set default can be string or unicode:
     if not isinstance(stylesheets, list):
-        stylesheets = [cls.strip() for cls in stylesheets.split(',')]
-    return stylesheets
+        stylesheets = [path.strip() for path in stylesheets.split(',')]
+    # expand relative paths if found in stylesheet-dirs:
+    return [find_file_in_dirs(path, settings.stylesheet_dirs)
+            for path in stylesheets]
+
+def find_file_in_dirs(path, dirs):
+    """
+    Search for `path` in the list of directories `dirs`.
+
+    Return the first expansion that matches an existing file.
+    """
+    if os.path.isabs(path):
+        return path
+    for d in dirs:
+        if d == '.':
+            f = path
+        else:
+            d = os.path.expanduser(d)
+            f = os.path.join(d, path)
+        if os.path.exists(f):
+            return f
+    return path
 
 def get_trim_footnote_ref_space(settings):
     """
@@ -697,7 +717,7 @@ class DependencyList(object):
                 of = None
             else:
                 of = output_file
-            self.file = FileOutput(destination_path=of,
+            self.file = docutils.io.FileOutput(destination_path=of,
                                    encoding='utf8', autoclose=False)
         else:
             self.file = None
