@@ -1,5 +1,6 @@
-# $Id: __init__.py 7061 2011-06-29 16:24:09Z milde $
-# Author: David Goodger <goodger@python.org>
+# $Id: __init__.py 7661 2013-05-07 10:52:59Z milde $
+# Author: David Goodger
+# Maintainer: docutils-develop@lists.sourceforge.net
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -19,16 +20,22 @@ import os
 import os.path
 import time
 import re
-try:
-    import Image                        # check for the Python Imaging Library
+import urllib
+try: # check for the Python Imaging Library
+    import PIL.Image
 except ImportError:
-    Image = None
+    try:  # sometimes PIL modules are put in PYTHONPATH's root
+        import Image
+        class PIL(object): pass  # dummy wrapper
+        PIL.Image = Image
+    except ImportError:
+        PIL = None
 import docutils
 from docutils import frontend, nodes, utils, writers, languages, io
+from docutils.utils.error_reporting import SafeString
 from docutils.transforms import writer_aux
-from docutils.math import unimathsymbols2tex, pick_math_environment
-from docutils.math.latex2mathml import parse_latex_math
-from docutils.math.math2html import math2html
+from docutils.utils.math import unichar2tex, pick_math_environment, math2html
+from docutils.utils.math.latex2mathml import parse_latex_math
 
 class Writer(writers.Writer):
 
@@ -36,10 +43,8 @@ class Writer(writers.Writer):
     """Formats this writer supports."""
 
     default_stylesheet = 'html4css1.css'
-
-    default_stylesheet_path = utils.relative_path(
-        os.path.join(os.getcwd(), 'dummy'),
-        os.path.join(os.path.dirname(__file__), default_stylesheet))
+    default_stylesheet_dirs = ['.', utils.relative_path(
+        os.path.join(os.getcwd(), 'dummy'), os.path.dirname(__file__))]
 
     default_template = 'template.txt'
 
@@ -54,17 +59,20 @@ class Writer(writers.Writer):
           % default_template_path,
           ['--template'],
           {'default': default_template_path, 'metavar': '<file>'}),
-        ('Specify comma separated list of stylesheet URLs. '
+         ('Comma separated list of stylesheet URLs. '
           'Overrides previous --stylesheet and --stylesheet-path settings.',
           ['--stylesheet'],
-          {'metavar': '<URL>', 'overrides': 'stylesheet_path'}),
-         ('Specify comma separated list of stylesheet paths. '
-          'With --link-stylesheet, '
+          {'metavar': '<URL[,URL,...]>', 'overrides': 'stylesheet_path',
+           'validator': frontend.validate_comma_separated_list}),
+         ('Comma separated list of stylesheet paths. '
+          'Relative paths are expanded if a matching file is found in '
+          'the --stylesheet-dirs. With --link-stylesheet, '
           'the path is rewritten relative to the output HTML file. '
-          'Default: "%s"' % default_stylesheet_path,
+          'Default: "%s"' % default_stylesheet,
           ['--stylesheet-path'],
-          {'metavar': '<file>', 'overrides': 'stylesheet',
-           'default': default_stylesheet_path}),
+          {'metavar': '<file[,file,...]>', 'overrides': 'stylesheet',
+           'validator': frontend.validate_comma_separated_list,
+           'default': [default_stylesheet]}),
          ('Embed the stylesheet(s) in the output HTML file.  The stylesheet '
           'files must be accessible during processing. This is the default.',
           ['--embed-stylesheet'],
@@ -74,6 +82,13 @@ class Writer(writers.Writer):
           'Default: embed stylesheets.',
           ['--link-stylesheet'],
           {'dest': 'embed_stylesheet', 'action': 'store_false'}),
+         ('Comma-separated list of directories where stylesheets are found. '
+          'Used by --stylesheet-path when expanding relative path arguments. '
+          'Default: "%s"' % default_stylesheet_dirs,
+          ['--stylesheet-dirs'],
+          {'metavar': '<dir[,dir,...]>',
+           'validator': frontend.validate_comma_separated_list,
+           'default': default_stylesheet_dirs}),
          ('Specify the initial header level.  Default is 1 for "<h1>".  '
           'Does not affect document title & subtitle (see --no-doc-title).',
           ['--initial-header-level'],
@@ -125,9 +140,9 @@ class Writer(writers.Writer):
           ['--table-style'],
           {'default': ''}),
          ('Math output format, one of "MathML", "HTML", "MathJax" '
-          'or "LaTeX". Default: "MathML"',
+          'or "LaTeX". Default: "HTML math.css"',
           ['--math-output'],
-          {'default': 'MathML'}),
+          {'default': 'HTML math.css'}),
          ('Omit the XML declaration.  Use with caution.',
           ['--no-xml-declaration'],
           {'dest': 'xml_declaration', 'default': 1, 'action': 'store_false',
@@ -139,8 +154,6 @@ class Writer(writers.Writer):
 
     settings_defaults = {'output_encoding_error_handler': 'xmlcharrefreplace'}
 
-    relative_path_settings = ('stylesheet_path',)
-
     config_section = 'html4css1 writer'
     config_section_dependencies = ('writers',)
 
@@ -245,7 +258,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     generator = ('<meta name="generator" content="Docutils %s: '
                  'http://docutils.sourceforge.net/" />\n')
-    
+
     # Template for the MathJax script in the header:
     mathjax_script = '<script type="text/javascript" src="%s"></script>\n'
     # The latest version of MathJax from the distributed server:
@@ -253,12 +266,7 @@ class HTMLTranslator(nodes.NodeVisitor):
     # __http://www.mathjax.org/download/mathjax-cdn-terms-of-service/
     mathjax_url = ('http://cdn.mathjax.org/mathjax/latest/MathJax.js?'
                    'config=TeX-AMS-MML_HTMLorMML')
-    # TODO: make this configurable:
-    #
-    # a) as extra option or
-    # b) appended to math-output="MathJax"?
-    #
-    # If b), which delimiter/delimter-set (':', ',', ' ')? 
+    # may be overwritten by custom URL appended to "mathjax"
 
     stylesheet_link = '<link rel="stylesheet" href="%s" type="text/css" />\n'
     embedded_stylesheet = '<style type="text/css">\n\n%s\n</style>\n'
@@ -280,19 +288,8 @@ class HTMLTranslator(nodes.NodeVisitor):
             # encoding not interpolated:
             self.html_prolog.append(self.xml_declaration)
         self.head = self.meta[:]
-        # stylesheets
-        styles = utils.get_stylesheet_list(settings)
-        if settings.stylesheet_path and not(settings.embed_stylesheet):
-            styles = [utils.relative_path(settings._destination, sheet)
-                      for sheet in styles]
-        if settings.embed_stylesheet:
-            settings.record_dependencies.add(*styles)
-            self.stylesheet = [self.embedded_stylesheet %
-                io.FileInput(source_path=sheet, encoding='utf-8').read()
-                for sheet in styles]
-        else: # link to stylesheets
-            self.stylesheet = [self.stylesheet_link % self.encode(stylesheet)
-                               for stylesheet in styles]
+        self.stylesheet = [self.stylesheet_call(path)
+                           for path in utils.get_stylesheet_list(settings)]
         self.body_prefix = ['</head>\n<body>\n']
         # document title, subtitle display
         self.body_pre_docinfo = []
@@ -303,17 +300,21 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body_suffix = ['</body>\n</html>\n']
         self.section_level = 0
         self.initial_header_level = int(settings.initial_header_level)
-        self.math_output = settings.math_output.lower()
+
+        self.math_output = settings.math_output.split()
+        self.math_output_options = self.math_output[1:]
+        self.math_output = self.math_output[0].lower()
+
         # A heterogenous stack used in conjunction with the tree traversal.
         # Make sure that the pops correspond to the pushes:
         self.context = []
         self.topic_classes = []
         self.colspecs = []
-        self.compact_p = 1
-        self.compact_simple = None
-        self.compact_field_list = None
-        self.in_docinfo = None
-        self.in_sidebar = None
+        self.compact_p = True
+        self.compact_simple = False
+        self.compact_field_list = False
+        self.in_docinfo = False
+        self.in_sidebar = False
         self.title = []
         self.subtitle = []
         self.header = []
@@ -322,10 +323,10 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.html_title = []
         self.html_subtitle = []
         self.html_body = []
-        self.in_document_title = 0
-        self.in_mailto = 0
-        self.author_in_authors = None
-        self.math_header = ''
+        self.in_document_title = 0   # len(self.body) or 0
+        self.in_mailto = False
+        self.author_in_authors = False
+        self.math_header = []
 
     def astext(self):
         return ''.join(self.head_prefix + self.head
@@ -371,7 +372,26 @@ class HTMLTranslator(nodes.NodeVisitor):
             encoded = encoded.replace('.', '&#46;')
         return encoded
 
-    def starttag(self, node, tagname, suffix='\n', empty=0, **attributes):
+    def stylesheet_call(self, path):
+        """Return code to reference or embed stylesheet file `path`"""
+        if self.settings.embed_stylesheet:
+            try:
+                content = io.FileInput(source_path=path,
+                                       encoding='utf-8').read()
+                self.settings.record_dependencies.add(path)
+            except IOError, err:
+                msg = u"Cannot embed stylesheet '%s': %s." % (
+                                path, SafeString(err.strerror))
+                self.document.reporter.error(msg)
+                return '<--- %s --->\n' % msg
+            return self.embedded_stylesheet % content
+        # else link to style file:
+        if self.settings.stylesheet_path:
+            # adapt path relative to output (cf. config.html#stylesheet-path)
+            path = utils.relative_path(self.settings._destination, path)
+        return self.stylesheet_link % self.encode(path)
+
+    def starttag(self, node, tagname, suffix='\n', empty=False, **attributes):
         """
         Construct and return a start tag given a node (id & class attributes
         are extracted), tag name, and optional attributes.
@@ -382,19 +402,19 @@ class HTMLTranslator(nodes.NodeVisitor):
         ids = []
         for (name, value) in attributes.items():
             atts[name.lower()] = value
-        classes = node.get('classes', [])
-        if 'class' in atts:
-            classes.append(atts.pop('class'))
-        # move language specification to 'lang' attribute
-        languages = [cls for cls in classes
-                     if cls.startswith('language-')]
+        classes = []
+        languages = []
+        # unify class arguments and move language specification
+        for cls in node.get('classes', []) + atts.pop('class', '').split() :
+            if cls.startswith('language-'):
+                languages.append(cls[9:])
+            elif cls.strip() and cls not in classes:
+                classes.append(cls)
         if languages:
             # attribute name is 'lang' in XHTML 1.0 but 'xml:lang' in 1.1
-            atts[self.lang_attribute] = languages[0][9:]
-            classes.pop(classes.index(languages[0]))
-        classes = ' '.join(classes).strip()
+            atts[self.lang_attribute] = languages[0]
         if classes:
-            atts['class'] = classes
+            atts['class'] = ' '.join(classes)
         assert 'id' not in atts
         ids.extend(node.get('ids', []))
         if 'ids' in atts:
@@ -437,7 +457,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def emptytag(self, node, tagname, suffix='\n', **attributes):
         """Construct and return an XML-compatible empty tag."""
-        return self.starttag(node, tagname, suffix, empty=1, **attributes)
+        return self.starttag(node, tagname, suffix, empty=True, **attributes)
 
     def set_class_on_child(self, node, class_, index=0):
         """
@@ -480,7 +500,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append('</acronym>')
 
     def visit_address(self, node):
-        self.visit_docinfo_item(node, 'address', meta=None)
+        self.visit_docinfo_item(node, 'address', meta=False)
         self.body.append(self.starttag(node, 'pre', CLASS='address'))
 
     def depart_address(self, node):
@@ -517,17 +537,16 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def depart_author(self, node):
         if isinstance(node.parent, nodes.authors):
-            self.author_in_authors += 1
+            self.author_in_authors = True
         else:
             self.depart_docinfo_item()
 
     def visit_authors(self, node):
         self.visit_docinfo_item(node, 'authors')
-        self.author_in_authors = 0      # initialize counter
+        self.author_in_authors = False  # initialize
 
     def depart_authors(self, node):
         self.depart_docinfo_item()
-        self.author_in_authors = None
 
     def visit_block_quote(self, node):
         self.body.append(self.starttag(node, 'blockquote'))
@@ -587,7 +606,13 @@ class HTMLTranslator(nodes.NodeVisitor):
                          '</tbody>\n</table>\n')
 
     def visit_citation_reference(self, node):
-        href = '#' + node['refid']
+        href = '#'
+        if 'refid' in node:
+            href += node['refid']
+        elif 'refname' in node:
+            href += self.document.nameids[node['refname']]
+        # else: # TODO system message (or already in the transform)?
+        # 'Citation reference missing.'
         self.body.append(self.starttag(
             node, 'a', '[', CLASS='citation-reference', href=href))
 
@@ -644,7 +669,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append('</div>\n')
 
     def visit_contact(self, node):
-        self.visit_docinfo_item(node, 'contact', meta=None)
+        self.visit_docinfo_item(node, 'contact', meta=False)
 
     def depart_contact(self, node):
         self.depart_docinfo_item()
@@ -702,16 +727,16 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append('<col class="docinfo-name" />\n'
                          '<col class="docinfo-content" />\n'
                          '<tbody valign="top">\n')
-        self.in_docinfo = 1
+        self.in_docinfo = True
 
     def depart_docinfo(self, node):
         self.body.append('</tbody>\n</table>\n')
-        self.in_docinfo = None
+        self.in_docinfo = False
         start = self.context.pop()
         self.docinfo = self.body[start:]
         self.body = []
 
-    def visit_docinfo_item(self, node, name, meta=1):
+    def visit_docinfo_item(self, node, name, meta=True):
         if meta:
             meta_tag = '<meta name="%s" content="%s" />\n' \
                        % (name, self.attval(node.astext()))
@@ -746,7 +771,10 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.meta.insert(0, self.content_type % self.settings.output_encoding)
         self.head.insert(0, self.content_type % self.settings.output_encoding)
         if self.math_header:
-            self.head.append(self.math_header)
+            if self.math_output == 'mathjax':
+                self.head.extend(self.math_header)
+            else:
+                self.stylesheet.extend(self.math_header)
         # skip content-type meta tag with interpolated charset value:
         self.html_head.extend(self.head[1:])
         self.body_prefix.append(self.starttag(node, 'div', CLASS='document'))
@@ -841,10 +869,10 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.context.append((self.compact_field_list, self.compact_p))
         self.compact_p = None
         if 'compact' in node['classes']:
-            self.compact_field_list = 1
+            self.compact_field_list = True
         elif (self.settings.compact_field_lists
               and 'open' not in node['classes']):
-            self.compact_field_list = 1
+            self.compact_field_list = True
         if self.compact_field_list:
             for field in node:
                 field_body = field[-1]
@@ -855,7 +883,7 @@ class HTMLTranslator(nodes.NodeVisitor):
                         len(children) == 1 and
                         isinstance(children[0],
                                    (nodes.paragraph, nodes.line_block))):
-                    self.compact_field_list = 0
+                    self.compact_field_list = False
                     break
         self.body.append(self.starttag(node, 'table', frame='void',
                                        rules='none',
@@ -878,7 +906,8 @@ class HTMLTranslator(nodes.NodeVisitor):
              and len(node.astext()) > self.settings.field_name_limit):
             atts['colspan'] = 2
             self.context.append('</tr>\n'
-                                + self.starttag(node.parent, 'tr', '')
+                                + self.starttag(node.parent, 'tr', '', 
+                                                CLASS='field')
                                 + '<td>&nbsp;</td>')
         else:
             self.context.append('')
@@ -1006,18 +1035,22 @@ class HTMLTranslator(nodes.NodeVisitor):
         if 'height' in node:
             atts['height'] = node['height']
         if 'scale' in node:
-            if Image and not ('width' in node and 'height' in node):
+            if (PIL and not ('width' in node and 'height' in node)
+                and self.settings.file_insertion_enabled):
+                imagepath = urllib.url2pathname(uri)
                 try:
-                    im = Image.open(str(uri))
-                except (IOError, # Source image can't be found or opened
-                        UnicodeError):  # PIL doesn't like Unicode paths.
-                    pass
+                    img = PIL.Image.open(
+                            imagepath.encode(sys.getfilesystemencoding()))
+                except (IOError, UnicodeEncodeError):
+                    pass # TODO: warn?
                 else:
+                    self.settings.record_dependencies.add(
+                        imagepath.replace('\\', '/'))
                     if 'width' not in atts:
-                        atts['width'] = str(im.size[0])
+                        atts['width'] = str(img.size[0])
                     if 'height' not in atts:
-                        atts['height'] = str(im.size[1])
-                    del im
+                        atts['height'] = str(img.size[1])
+                    del img
             for att_name in 'width', 'height':
                 if att_name in atts:
                     match = re.match(r'([0-9.]+)(\S*)$', atts[att_name])
@@ -1099,7 +1132,13 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append('</li>\n')
 
     def visit_literal(self, node):
-        """Process text to prevent tokens from wrapping."""
+        # special case: "code" role
+        classes = node.get('classes', [])
+        if 'code' in classes:
+            # filter 'code' from class arguments
+            node['classes'] = [cls for cls in classes if cls != 'code']
+            self.body.append(self.starttag(node, 'code', ''))
+            return
         self.body.append(
             self.starttag(node, 'tt', '', CLASS='docutils literal'))
         text = node.astext()
@@ -1122,6 +1161,10 @@ class HTMLTranslator(nodes.NodeVisitor):
         # Content already processed:
         raise nodes.SkipNode
 
+    def depart_literal(self, node):
+        # skipped unless literal element is from "code" role:
+        self.body.append('</code>')
+
     def visit_literal_block(self, node):
         self.body.append(self.starttag(node, 'pre', CLASS='literal-block'))
 
@@ -1129,10 +1172,16 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append('\n</pre>\n')
 
     def visit_math(self, node, math_env=''):
+        # If the method is called from visit_math_block(), math_env != ''.
+
         # As there is no native HTML math support, we provide alternatives:
         # LaTeX and MathJax math_output modes simply wrap the content,
         # HTML and MathML math_output modes also convert the math_code.
-        # If the method is called from visit_math_block(), math_env != ''.
+        if self.math_output not in ('mathml', 'html', 'mathjax', 'latex'):
+            self.document.reporter.error(
+                'math-output format "%s" not supported '
+                'falling back to "latex"'% self.math_output)
+            self.math_output = 'latex'
         #
         # HTML container
         tags = {# math_output: (block, inline, class-arguments)
@@ -1152,7 +1201,7 @@ class HTMLTranslator(nodes.NodeVisitor):
                    }
         wrapper = wrappers[self.math_output][math_env != '']
         # get and wrap content
-        math_code = node.astext().translate(unimathsymbols2tex.uni2tex_table)
+        math_code = node.astext().translate(unichar2tex.uni2tex_table)
         if wrapper and math_env:
             math_code = wrapper % (math_env, math_code, math_env)
         elif wrapper:
@@ -1160,10 +1209,18 @@ class HTMLTranslator(nodes.NodeVisitor):
         # settings and conversion
         if self.math_output in ('latex', 'mathjax'):
             math_code = self.encode(math_code)
-        if self.math_output == 'mathjax':
-            self.math_header = self.mathjax_script % self.mathjax_url
+        if self.math_output == 'mathjax' and not self.math_header:
+            if self.math_output_options:
+                self.mathjax_url = self.math_output_options[0]
+            self.math_header = [self.mathjax_script % self.mathjax_url]
         elif self.math_output == 'html':
-            math_code = math2html(math_code)
+            if self.math_output_options and not self.math_header:
+                self.math_header = [self.stylesheet_call(
+                    utils.find_file_in_dirs(s, self.settings.stylesheet_dirs))
+                    for s in self.math_output_options[0].split(',')]
+            # TODO: fix display mode in matrices and fractions
+            math2html.DocumentParameters.displaymode = (math_env != '')
+            math_code = math2html.math2html(math_code)
         elif self.math_output == 'mathml':
             self.doctype = self.doctype_mathml
             self.content_type = self.content_type_mathml
@@ -1176,7 +1233,7 @@ class HTMLTranslator(nodes.NodeVisitor):
                 self.body.append(self.starttag(node, 'p'))
                 self.body.append(u','.join(err.args))
                 self.body.append('</p>\n')
-                self.body.append(self.starttag(node, 'pre', 
+                self.body.append(self.starttag(node, 'pre',
                                                CLASS='literal-block'))
                 self.body.append(self.encode(math_code))
                 self.body.append('\n</pre>\n')
@@ -1184,7 +1241,9 @@ class HTMLTranslator(nodes.NodeVisitor):
                 raise nodes.SkipNode
         # append to document body
         if tag:
-            self.body.append(self.starttag(node, tag, CLASS=clsarg))
+            self.body.append(self.starttag(node, tag,
+                                           suffix='\n'*bool(math_env),
+                                           CLASS=clsarg))
         self.body.append(math_code)
         if math_env:
             self.body.append('\n')
@@ -1197,6 +1256,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         pass # never reached
 
     def visit_math_block(self, node):
+        # print node.astext().encode('utf8')
         math_env = pick_math_environment(node.astext())
         self.visit_math(node, math_env=math_env)
 
@@ -1284,13 +1344,13 @@ class HTMLTranslator(nodes.NodeVisitor):
         if (isinstance(node.parent, nodes.document) or
             isinstance(node.parent, nodes.compound)):
             # Never compact paragraphs in document or compound.
-            return 0
+            return False
         for key, value in node.attlist():
             if (node.is_not_default(key) and
                 not (key == 'classes' and value in
                      ([], ['first'], ['last'], ['first', 'last']))):
                 # Attribute which needs to survive.
-                return 0
+                return False
         first = isinstance(node.parent[0], nodes.label) # skip label
         for child in node.parent.children[first:]:
             # only first paragraph can be compact
@@ -1298,14 +1358,14 @@ class HTMLTranslator(nodes.NodeVisitor):
                 continue
             if child is node:
                 break
-            return 0
+            return False
         parent_length = len([n for n in node.parent if not isinstance(
             n, (nodes.Invisible, nodes.label))])
         if ( self.compact_simple
              or self.compact_field_list
              or self.compact_p and parent_length == 1):
-            return 1
-        return 0
+            return True
+        return False
 
     def visit_paragraph(self, node):
         if self.should_be_compact_paragraph(node):
@@ -1347,7 +1407,7 @@ class HTMLTranslator(nodes.NodeVisitor):
             if ( self.settings.cloak_email_addresses
                  and atts['href'].startswith('mailto:')):
                 atts['href'] = self.cloak_mailto(atts['href'])
-                self.in_mailto = 1
+                self.in_mailto = True
             atts['class'] += ' external'
         else:
             assert 'refid' in node, \
@@ -1363,10 +1423,10 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append('</a>')
         if not isinstance(node.parent, nodes.TextElement):
             self.body.append('\n')
-        self.in_mailto = 0
+        self.in_mailto = False
 
     def visit_revision(self, node):
-        self.visit_docinfo_item(node, 'revision', meta=None)
+        self.visit_docinfo_item(node, 'revision', meta=False)
 
     def depart_revision(self, node):
         self.depart_docinfo_item()
@@ -1397,14 +1457,14 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append(
             self.starttag(node, 'div', CLASS='sidebar'))
         self.set_first_last(node)
-        self.in_sidebar = 1
+        self.in_sidebar = True
 
     def depart_sidebar(self, node):
         self.body.append('</div>\n')
-        self.in_sidebar = None
+        self.in_sidebar = False
 
     def visit_status(self, node):
-        self.visit_docinfo_item(node, 'status', meta=None)
+        self.visit_docinfo_item(node, 'status', meta=False)
 
     def depart_status(self, node):
         self.depart_docinfo_item()
@@ -1489,11 +1549,14 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append('</div>\n')
 
     def visit_table(self, node):
+        self.context.append(self.compact_p)
+        self.compact_p = True
         classes = ' '.join(['docutils', self.settings.table_style]).strip()
         self.body.append(
             self.starttag(node, 'table', CLASS=classes, border="1"))
 
     def depart_table(self, node):
+        self.compact_p = self.context.pop()
         self.body.append('</table>\n')
 
     def visit_target(self, node):
@@ -1547,7 +1610,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_title(self, node):
         """Only 6 section levels are supported by HTML."""
-        check_id = 0
+        check_id = 0  # TODO: is this a bool (False) or a counter?
         close_tag = '</p>\n'
         if isinstance(node.parent, nodes.topic):
             self.body.append(
@@ -1616,7 +1679,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         pass
 
     def visit_version(self, node):
-        self.visit_docinfo_item(node, 'version', meta=None)
+        self.visit_docinfo_item(node, 'version', meta=False)
 
     def depart_version(self, node):
         self.depart_docinfo_item()
