@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# $Id: package_unittest.py 4564 2006-05-21 20:44:42Z wiemann $
+# $Id: package_unittest.py 7320 2012-01-19 22:33:02Z milde $
 # Author: Garth Kidd <garth@deadlybloodyserious.com>
 # Copyright: This module has been placed in the public domain.
 
@@ -21,7 +21,7 @@ import re
 # So that individual test modules can share a bit of state,
 # `package_unittest` acts as an intermediary for the following
 # variables:
-debug = 0
+debug = False
 verbosity = 1
 
 USAGE = """\
@@ -116,8 +116,7 @@ def loadTestModules(path, name='', packages=None):
                 continue
             if type(suite) == types.FunctionType:
                 testSuite.addTest(suite())
-            elif type(suite) == types.InstanceType \
-                  and isinstance(suite, unittest.TestSuite):
+            elif isinstance(suite, unittest.TestSuite):
                 testSuite.addTest(suite)
             else:
                 raise AssertionError, "don't understand suite (%s)" % mod
@@ -156,4 +155,4 @@ def main(suite=None):
         for s in suite:
             testRunner.run(s)
     else:
-        testRunner.run(suite)
+        return testRunner.run(suite)
