@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# $Id: test_functional.py 5925 2009-04-27 10:50:38Z milde $
+# $Id: test_functional.py 7539 2012-11-26 13:50:06Z milde $
 # Author: Lea Wiemann <LeWiemann@gmail.com>
 # Copyright: This module has been placed in the public domain.
 
@@ -123,7 +123,7 @@ expected output and check it in:
         namespace = {}
         # Initialize 'settings_overrides' for test settings scripts,
         # and disable configuration files:
-        namespace['settings_overrides'] = {'_disable_config': 1}
+        namespace['settings_overrides'] = {'_disable_config': True}
         # Read the variables set in the default config file and in
         # the current config file into namespace:
         defaultpy = open(join_path(datadir, 'tests', '_default.py')).read()
@@ -157,27 +157,45 @@ expected output and check it in:
         # Get output (automatically written to the output/ directory
         # by publish_file):
         output = docutils.core.publish_file(**params)
+        # ensure output is unicode
+        output_encoding = params.get('output_encoding', 'utf-8')
+        if sys.version_info < (3,0):
+            try:
+                output = output.decode(output_encoding)
+            except UnicodeDecodeError:
+                # failsafe
+                output = output.decode('latin1', 'replace')
+        # Normalize line endings:
+        output = '\n'.join(output.splitlines())
         # Get the expected output *after* writing the actual output.
         no_expected = self.no_expected_template % {
             'exp': expected_path, 'out': params['destination_path']}
-        self.assert_(os.access(expected_path, os.R_OK), no_expected)
-        f = open(expected_path, 'rb')
-        expected = f.read()
+        self.assertTrue(os.access(expected_path, os.R_OK), no_expected)
+        if sys.version_info < (3,0):
+            f = open(expected_path, 'r')
+        else: # samples are UTF8 encoded. 'rb' leads to errors with Python 3!
+            f = open(expected_path, 'r', encoding='utf-8')
+        # Normalize line endings:
+        expected = '\n'.join(f.read().splitlines())
         f.close()
+        if sys.version_info < (3,0):
+            try:
+                expected = expected.decode(output_encoding)
+            except UnicodeDecodeError:
+                expected = expected.decode('latin1', 'replace')
+
         diff = self.expected_output_differs_template % {
             'exp': expected_path, 'out': params['destination_path']}
         try:
-            self.assertEquals(output, expected, diff)
+            self.assertEqual(output, expected, diff)
         except AssertionError:
-            if hasattr(difflib, 'unified_diff'):
-                # Generate diff if unified_diff available:
-                diff = ''.join(
-                    difflib.unified_diff(expected.decode('latin1').splitlines(1),
-                                         output.decode('latin1').splitlines(1),
-                                         expected_path,
-                                         params['destination_path']))
+            diff = ''.join(difflib.unified_diff(
+                expected.splitlines(True), output.splitlines(True),
+                expected_path, params['destination_path']))
+            if sys.version_info < (3,0):
+                diff = diff.encode(sys.stderr.encoding or 'ascii', 'replace')
             print >>sys.stderr, '\n%s:' % (self,)
-            print >>sys.stderr, diff.encode('ascii', 'replace')
+            print >>sys.stderr, diff
             raise
         # Execute optional function containing extra tests:
         if '_test_more' in namespace:
