@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# $Id: test_nodes.py 7463 2012-06-22 19:49:51Z milde $
+# $Id: test_nodes.py 7595 2013-01-21 17:33:56Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -33,6 +33,11 @@ class TextTests(unittest.TestCase):
         self.assertEqual(repr(self.text), r"<#text: 'Line 1.\nLine 2.'>")
         self.assertEqual(self.text.shortrepr(),
                           r"<#text: 'Line 1.\nLine 2.'>")
+        self.assertEqual(nodes.reprunicode('foo'), u'foo')
+        if sys.version_info < (3,):
+            self.assertEqual(repr(self.unicode_text), r"<#text: 'M\xf6hren'>")
+        else:
+            self.assertEqual(repr(self.unicode_text), u"<#text: 'Möhren'>")
 
     def test_str(self):
         self.assertEqual(str(self.text), 'Line 1.\nLine 2.')
@@ -42,10 +47,13 @@ class TextTests(unittest.TestCase):
         self.assertEqual(str(self.unicode_text), 'M\xf6hren')
 
     def test_astext(self):
-        self.assertEqual(self.text.astext(), 'Line 1.\nLine 2.')
+        self.assertTrue(isinstance(self.text.astext(), unicode))
+        self.assertEqual(self.text.astext(), u'Line 1.\nLine 2.')
+        self.assertEqual(self.unicode_text.astext(), u'Möhren')
 
     def test_pformat(self):
-        self.assertEqual(self.text.pformat(), 'Line 1.\nLine 2.\n')
+        self.assertTrue(isinstance(self.text.pformat(), unicode))
+        self.assertEqual(self.text.pformat(), u'Line 1.\nLine 2.\n')
 
     def test_asciirestriction(self):
         if sys.version_info < (3,):
@@ -88,11 +96,25 @@ class ElementTests(unittest.TestCase):
         dom = element.asdom()
         self.assertEqual(dom.toxml(), u'<Element mark="\u2022"/>')
         dom.unlink()
+        element['names'] = ['nobody', u'имя', u'näs']
+        if sys.version_info < (3,):
+            self.assertEqual(repr(element),
+                '<Element "nobody; \\u0438\\u043c\\u044f; n\\xe4s": >')
+        else:
+            self.assertEqual(repr(element), u'<Element "nobody; имя; näs": >')
+        self.assertTrue(isinstance(repr(element), str))
 
     def test_withtext(self):
         element = nodes.Element('text\nmore', nodes.Text('text\nmore'))
+        uelement = nodes.Element(u'grün', nodes.Text(u'grün'))
         self.assertEqual(repr(element), r"<Element: <#text: 'text\nmore'>>")
+        if sys.version_info < (3,):
+            self.assertEqual(repr(uelement), "<Element: <#text: 'gr\\xfcn'>>")
+        else:
+            self.assertEqual(repr(uelement), u"<Element: <#text: 'grün'>>")
+        self.assertTrue(isinstance(repr(uelement),str))
         self.assertEqual(str(element), '<Element>text\nmore</Element>')
+        self.assertEqual(str(uelement), '<Element>gr\xfcn</Element>')
         dom = element.asdom()
         self.assertEqual(dom.toxml(), '<Element>text\nmore</Element>')
         dom.unlink()
@@ -144,6 +166,117 @@ class ElementTests(unittest.TestCase):
         # 'test' is not overwritten because it is not a basic attribute.
         self.assertEqual(element1['test'], ['test1'])
 
+    def test_update_all_atts(self):
+        # Note: Also tests is_not_list_attribute and is_not_known_attribute
+        # and various helpers
+        ## Test for full attribute replacement
+        element1 = nodes.Element(ids=['foo', 'bar'], parent_only='parent',
+                                 all_nodes='mom')
+        element2 = nodes.Element(ids=['baz', 'qux'], child_only='child',
+                                 all_nodes='dad', source='source')
+
+        # Test for when same fields are replaced as well as source...
+        element1.update_all_atts_consistantly(element2, True, True)
+        # 'ids' are appended because 'ids' is a basic attribute.
+        self.assertEquals(element1['ids'], ['foo', 'bar', 'baz', 'qux'])
+        # 'parent_only' should remain unaffected.
+        self.assertEquals(element1['parent_only'], 'parent')
+        # 'all_nodes' is overwritten due to the second parameter == True.
+        self.assertEquals(element1['all_nodes'], 'dad')
+        # 'child_only' should have been added.
+        self.assertEquals(element1['child_only'], 'child')
+        # 'source' is also overwritten due to the third parameter == True.
+        self.assertEquals(element1['source'], 'source')
+
+        # Test for when same fields are replaced but not source...
+        element1 = nodes.Element(ids=['foo', 'bar'], parent_only='parent',
+                                 all_nodes='mom')
+        element1.update_all_atts_consistantly(element2)
+        # 'ids' are appended because 'ids' is a basic attribute.
+        self.assertEquals(element1['ids'], ['foo', 'bar', 'baz', 'qux'])
+        # 'parent_only' should remain unaffected.
+        self.assertEquals(element1['parent_only'], 'parent')
+        # 'all_nodes' is overwritten due to the second parameter default of True.
+        self.assertEquals(element1['all_nodes'], 'dad')
+        # 'child_only' should have been added.
+        self.assertEquals(element1['child_only'], 'child')
+        # 'source' remains unset due to the third parameter default of False.
+        self.assertEquals(element1.get('source'), None)
+
+        # Test for when fields are NOT replaced but source is...
+        element1 = nodes.Element(ids=['foo', 'bar'], parent_only='parent',
+                                 all_nodes='mom')
+        element1.update_all_atts_consistantly(element2, False, True)
+        # 'ids' are appended because 'ids' is a basic attribute.
+        self.assertEquals(element1['ids'], ['foo', 'bar', 'baz', 'qux'])
+        # 'parent_only' should remain unaffected.
+        self.assertEquals(element1['parent_only'], 'parent')
+        # 'all_nodes' is preserved due to the second parameter == False.
+        self.assertEquals(element1['all_nodes'], 'mom')
+        # 'child_only' should have been added.
+        self.assertEquals(element1['child_only'], 'child')
+        # 'source' is added due to the third parameter == True.
+        self.assertEquals(element1['source'], 'source')
+        element1 = nodes.Element(source='destination')
+        element1.update_all_atts_consistantly(element2, False, True)
+        # 'source' remains unchanged due to the second parameter == False.
+        self.assertEquals(element1['source'], 'destination')
+
+        # Test for when same fields are replaced but not source...
+        element1 = nodes.Element(ids=['foo', 'bar'], parent_only='parent',
+                                 all_nodes='mom')
+        element1.update_all_atts_consistantly(element2, False)
+        # 'ids' are appended because 'ids' is a basic attribute.
+        self.assertEquals(element1['ids'], ['foo', 'bar', 'baz', 'qux'])
+        # 'parent_only' should remain unaffected.
+        self.assertEquals(element1['parent_only'], 'parent')
+        # 'all_nodes' is preserved due to the second parameter == False.
+        self.assertEquals(element1['all_nodes'], 'mom')
+        # 'child_only' should have been added.
+        self.assertEquals(element1['child_only'], 'child')
+        # 'source' remains unset due to the third parameter default of False.
+        self.assertEquals(element1.get('source'), None)
+
+        ## Test for List attribute merging
+        # Attribute Concatination
+        element1 = nodes.Element(ss='a', sl='1', ls=['I'], ll=['A'])
+        element2 = nodes.Element(ss='b', sl=['2'], ls='II', ll=['B'])
+        element1.update_all_atts_concatenating(element2)
+        # 'ss' is replaced because non-list
+        self.assertEquals(element1['ss'], 'b')
+        # 'sl' is replaced because they are both not lists
+        self.assertEquals(element1['sl'], ['2'])
+        # 'ls' is replaced because they are both not lists
+        self.assertEquals(element1['ls'], 'II')
+        # 'll' is extended because they are both lists
+        self.assertEquals(element1['ll'], ['A', 'B'])
+
+        # Attribute Coercion
+        element1 = nodes.Element(ss='a', sl='1', ls=['I'], ll=['A'])
+        element2 = nodes.Element(ss='b', sl=['2'], ls='II', ll=['B'])
+        element1.update_all_atts_coercion(element2)
+        # 'ss' is replaced because non-list
+        self.assertEquals(element1['ss'], 'b')
+        # 'sl' is converted to a list and appended because element2 has a list
+        self.assertEquals(element1['sl'], ['1', '2'])
+        # 'ls' has element2's value appended to the list
+        self.assertEquals(element1['ls'], ['I', 'II'])
+        # 'll' is extended because they are both lists
+        self.assertEquals(element1['ll'], ['A', 'B'])
+
+        # Attribute Conversion
+        element1 = nodes.Element(ss='a', sl='1', ls=['I'], ll=['A'])
+        element2 = nodes.Element(ss='b', sl=['2'], ls='II', ll=['B'])
+        element1.update_all_atts_convert(element2)
+        # 'ss' is converted to a list with the values from each element
+        self.assertEquals(element1['ss'], ['a', 'b'])
+        # 'sl' is converted to a list and appended
+        self.assertEquals(element1['sl'], ['1', '2'])
+        # 'ls' has element2's value appended to the list
+        self.assertEquals(element1['ls'], ['I', 'II'])
+        # 'll' is extended
+        self.assertEquals(element1['ll'], ['A', 'B'])
+
     def test_replace_self(self):
         parent = nodes.Element(ids=['parent'])
         child1 = nodes.Element(ids=['child1'])
@@ -194,6 +327,25 @@ class ElementTests(unittest.TestCase):
 
 class MiscTests(unittest.TestCase):
 
+    def test_reprunicode(self):
+        # return `unicode` instance
+        self.assertTrue(isinstance(nodes.reprunicode('foo'), unicode))
+        self.assertEqual(nodes.reprunicode('foo'), u'foo')
+        self.assertEqual(nodes.reprunicode(u'Möhre'), u'Möhre')
+        if sys.version_info < (3,): # strip leading "u" from representation
+            self.assertEqual(repr(nodes.reprunicode(u'Möhre')),
+                             repr(u'Möhre')[1:])
+        else: # no change to `unicode` under Python 3k
+            self.assertEqual(repr(nodes.reprunicode(u'Möhre')), repr(u'Möhre'))
+
+    def test_ensure_str(self):
+        self.assertTrue(isinstance(nodes.ensure_str(u'über'), str))
+        self.assertEqual(nodes.ensure_str('over'), 'over')
+        if sys.version_info < (3,): # strip leading "u" from representation
+            self.assertEqual(nodes.ensure_str(u'über'), r'\xfcber')
+        else:
+            self.assertEqual(nodes.ensure_str(u'über'), r'über')
+
     def test_node_class_names(self):
         node_class_names = []
         for x in dir(nodes):
@@ -484,7 +636,7 @@ class TreeCopyVisitorTests(unittest.TestCase):
 
     def compare_trees(self, one, two):
         self.assertEqual(one.__class__, two.__class__)
-        self.assertNotEquals(id(one), id(two))
+        self.assertNotEqual(id(one), id(two))
         self.assertEqual(len(one.children), len(two.children))
         for i in range(len(one.children)):
             self.compare_trees(one.children[i], two.children[i])
@@ -534,7 +686,7 @@ class MiscFunctionTests(unittest.TestCase):
         element = nodes.Element()
         document.set_id(element)
         self.assertEqual(element['ids'], ['prefixauto1'])
-        
+
 
 if __name__ == '__main__':
     unittest.main()
