@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# $Id: test_tables.py 7463 2012-06-22 19:49:51Z milde $
+# $Id: test_tables.py 7664 2013-05-17 08:16:41Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -10,7 +10,7 @@ Tests for tables.py directives.
 
 from __init__ import DocutilsTestSupport
 
-import os
+import os, sys
 import csv
 from docutils.parsers.rst.directives import tables
 
@@ -32,6 +32,15 @@ if isinstance(unichr_exception, OverflowError):
 else:
     unichr_exception_string = str(unichr_exception)
 
+# some error messages changed in Python 3.3:
+csv_eod_error_str = 'unexpected end of data'
+if sys.version_info < (3,2,4) and not (# backport to 2.7.4
+    sys.version_info[:2] == (2,7) and sys.version_info[2] > 3):
+    csv_eod_error_str = 'newline inside string'
+csv_unknown_url = "'bogus.csv'"
+if sys.version_info < (3,3,2):
+    csv_unknown_url = "bogus.csv"
+
 def null_bytes():
     import csv
     csv_data = open(utf_16_csv, 'rb').read()
@@ -572,11 +581,11 @@ u"""\
     <system_message level="4" line="1" source="test data" type="SEVERE">
         <paragraph>
             Problems with "csv-table" directive URL "bogus.csv":
-            unknown url type: bogus.csv.
+            unknown url type: %s.
         <literal_block xml:space="preserve">
             .. csv-table:: bad URL
                :url: bogus.csv
-"""],
+""" % csv_unknown_url],
 ["""\
 .. csv-table:: column mismatch
    :widths: 10,20
@@ -760,12 +769,12 @@ u"""\
     <system_message level="3" line="1" source="test data" type="ERROR">
         <paragraph>
             Error with CSV data in "csv-table" directive:
-            newline inside string
+            %s
         <literal_block xml:space="preserve">
             .. csv-table:: bad CSV data
             \n\
                "bad", \"csv, data
-"""],
+""" % csv_eod_error_str],
 ["""\
 .. csv-table:: bad CSV header data
    :header: "bad", \"csv, data
@@ -777,13 +786,13 @@ u"""\
     <system_message level="3" line="1" source="test data" type="ERROR">
         <paragraph>
             Error with CSV data in "csv-table" directive:
-            newline inside string
+            %s
         <literal_block xml:space="preserve">
             .. csv-table:: bad CSV header data
                :header: "bad", \"csv, data
             \n\
                good, csv, data
-"""],
+""" % csv_eod_error_str],
 ["""\
 .. csv-table:: bad encoding
    :file: %s
