@@ -1,9 +1,7 @@
 #! /usr/bin/env python
 
-# Author: Felix Wiemann
-# Contact: Felix_Wiemann@ososo.de
-# Revision: $Revision: 4132 $
-# Date: $Date: 2005-12-03 03:13:12 +0100 (Sat, 03 Dec 2005) $
+# $Id: test_writer_aux.py 5174 2007-05-31 00:01:52Z wiemann $
+# Author: Lea Wiemann <LeWiemann@gmail.com>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -51,6 +49,39 @@ totest['compound'] = ((writer_aux.Compound,), [
 """],
 ])
 
+totest['admonitions'] = ((writer_aux.Admonitions,), [
+["""\
+.. note::
+
+   These are the note contents.
+
+   Another paragraph.
+""",
+"""\
+<document source="test data">
+    <admonition classes="note">
+        <title>
+            Note
+        <paragraph>
+            These are the note contents.
+        <paragraph>
+            Another paragraph.
+"""],
+["""\
+.. admonition:: Generic
+
+   Admonitions contents...
+""",
+"""\
+<document source="test data">
+    <admonition classes="admonition-generic admonition">
+        <title>
+            Generic
+        <paragraph>
+            Admonitions contents...
+"""],
+])
+
 
 if __name__ == '__main__':
     import unittest
